@@ -1,46 +1,30 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
-
 // Inspiration for this code comes from Salvatore Sanfilippo's linenoise.
 // https://github.com/antirez/linenoise
 // Reference:
 // * http://invisible-island.net/xterm/ctlseqs/ctlseqs.html
 // * http://www.3waylabs.com/nw/WWW/products/wizcon/vt220.html
 
-var kHistorySize = 30;
-var kBufSize = 10 * 1024;
+'use strict';
 
-var util = require('util');
-var inherits = require('util').inherits;
-var EventEmitter = require('events').EventEmitter;
+const kHistorySize = 30;
+const kMincrlfDelay = 100;
+const kMaxcrlfDelay = 2000;
+
+const util = require('util');
+const debug = util.debuglog('readline');
+const internalUtil = require('internal/util');
+const inherits = util.inherits;
+const Buffer = require('buffer').Buffer;
+const EventEmitter = require('events');
+const internalReadline = require('internal/readline');
+const emitKeys = internalReadline.emitKeys;
+const getStringWidth = internalReadline.getStringWidth;
+const isFullWidthCodePoint = internalReadline.isFullWidthCodePoint;
+const stripVTControlCharacters = internalReadline.stripVTControlCharacters;
 
 
 exports.createInterface = function(input, output, completer, terminal) {
-  var rl;
-  if (arguments.length === 1) {
-    rl = new Interface(input);
-  } else {
-    rl = new Interface(input, output, completer, terminal);
-  }
-  return rl;
+  return new Interface(input, output, completer, terminal);
 };
 
 
@@ -49,27 +33,45 @@ function Interface(input, output, completer, terminal) {
     return new Interface(input, output, completer, terminal);
   }
 
-  this._sawReturn = false;
+  this._sawReturnAt = 0;
+  this.isCompletionEnabled = true;
+  this._sawKeyPress = false;
 
   EventEmitter.call(this);
+  var historySize;
+  let crlfDelay;
+  let prompt = '> ';
 
-  if (arguments.length === 1) {
+  if (input && input.input) {
     // an options object was given
     output = input.output;
     completer = input.completer;
     terminal = input.terminal;
+    historySize = input.historySize;
+    if (input.prompt !== undefined) {
+      prompt = input.prompt;
+    }
+    crlfDelay = input.crlfDelay;
     input = input.input;
   }
 
-  completer = completer || function() { return []; };
+  if (completer && typeof completer !== 'function') {
+    throw new TypeError('Argument "completer" must be a function');
+  }
 
-  if (typeof completer !== 'function') {
-    throw new TypeError('Argument \'completer\' must be a function');
+  if (historySize === undefined) {
+    historySize = kHistorySize;
+  }
+
+  if (typeof historySize !== 'number' ||
+      isNaN(historySize) ||
+      historySize < 0) {
+    throw new TypeError('Argument "historySize" must be a positive number');
   }
 
   // backwards compat; check the isTTY prop of the output stream
   //  when `terminal` was not specified
-  if (typeof terminal == 'undefined') {
+  if (terminal === undefined && !(output === null || output === undefined)) {
     terminal = !!output.isTTY;
   }
 
@@ -77,13 +79,18 @@ function Interface(input, output, completer, terminal) {
 
   this.output = output;
   this.input = input;
+  this.historySize = historySize;
+  this.crlfDelay = Math.max(kMincrlfDelay,
+                            Math.min(kMaxcrlfDelay, crlfDelay >>> 0));
 
   // Check arity, 2 - for async, 1 for sync
-  this.completer = completer.length === 2 ? completer : function(v, callback) {
-    callback(null, completer(v));
-  };
+  if (typeof completer === 'function') {
+    this.completer = completer.length === 2 ? completer : function(v, cb) {
+      cb(null, completer(v));
+    };
+  }
 
-  this.setPrompt('> ');
+  this.setPrompt(prompt);
 
   this.terminal = !!terminal;
 
@@ -92,6 +99,17 @@ function Interface(input, output, completer, terminal) {
   }
 
   function onend() {
+    if (typeof self._line_buffer === 'string' &&
+        self._line_buffer.length > 0) {
+      self.emit('line', self._line_buffer);
+    }
+    self.close();
+  }
+
+  function ontermend() {
+    if (typeof self.line === 'string' && self.line.length > 0) {
+      self.emit('line', self.line);
+    }
     self.close();
   }
 
@@ -115,10 +133,11 @@ function Interface(input, output, completer, terminal) {
 
   } else {
 
-    exports.emitKeypressEvents(input);
+    emitKeypressEvents(input, this);
 
     // input usually refers to stdin
     input.on('keypress', onkeypress);
+    input.on('end', ontermend);
 
     // Current line
     this.line = '';
@@ -132,10 +151,15 @@ function Interface(input, output, completer, terminal) {
     this.history = [];
     this.historyIndex = -1;
 
-    output.on('resize', onresize);
+    if (output !== null && output !== undefined)
+      output.on('resize', onresize);
+
     self.once('close', function() {
       input.removeListener('keypress', onkeypress);
-      output.removeListener('resize', onresize);
+      input.removeListener('end', ontermend);
+      if (output !== null && output !== undefined) {
+        output.removeListener('resize', onresize);
+      }
     });
   }
 
@@ -144,26 +168,30 @@ function Interface(input, output, completer, terminal) {
 
 inherits(Interface, EventEmitter);
 
-Interface.prototype.__defineGetter__('columns', function() {
-  return this.output.columns || Infinity;
+Object.defineProperty(Interface.prototype, 'columns', {
+  configurable: true,
+  enumerable: true,
+  get: function() {
+    var columns = Infinity;
+    if (this.output && this.output.columns)
+      columns = this.output.columns;
+    return columns;
+  }
 });
 
-Interface.prototype.setPrompt = function(prompt, length) {
+Interface.prototype.setPrompt = function(prompt) {
   this._prompt = prompt;
-  if (length) {
-    this._promptLength = length;
-  } else {
-    var lines = prompt.split(/[\r\n]/);
-    var lastLine = lines[lines.length - 1];
-    this._promptLength = lastLine.length;
-  }
 };
 
 
 Interface.prototype._setRawMode = function(mode) {
+  const wasInRawMode = this.input.isRaw;
+
   if (typeof this.input.setRawMode === 'function') {
-    return this.input.setRawMode(mode);
+    this.input.setRawMode(mode);
   }
+
+  return wasInRawMode;
 };
 
 
@@ -173,7 +201,7 @@ Interface.prototype.prompt = function(preserveCursor) {
     if (!preserveCursor) this.cursor = 0;
     this._refreshLine();
   } else {
-    this.output.write(this._prompt);
+    this._writeToOutput(this._prompt);
   }
 };
 
@@ -203,15 +231,28 @@ Interface.prototype._onLine = function(line) {
   }
 };
 
+Interface.prototype._writeToOutput = function _writeToOutput(stringToWrite) {
+  if (typeof stringToWrite !== 'string')
+    throw new TypeError('"stringToWrite" argument must be a string');
+
+  if (this.output !== null && this.output !== undefined)
+    this.output.write(stringToWrite);
+};
 
 Interface.prototype._addHistory = function() {
   if (this.line.length === 0) return '';
 
+  // if the history is disabled then return the line
+  if (this.historySize === 0) return this.line;
+
+  // if the trimmed line is empty then return the line
+  if (this.line.trim().length === 0) return this.line;
+
   if (this.history.length === 0 || this.history[0] !== this.line) {
     this.history.unshift(this.line);
 
     // Only store so many
-    if (this.history.length > kHistorySize) this.history.pop();
+    if (this.history.length > this.historySize) this.history.pop();
   }
 
   this.historyIndex = -1;
@@ -220,13 +261,11 @@ Interface.prototype._addHistory = function() {
 
 
 Interface.prototype._refreshLine = function() {
-  var columns = this.columns;
-
   // line length
   var line = this._prompt + this.line;
-  var lineLength = line.length;
-  var lineCols = lineLength % columns;
-  var lineRows = (lineLength - lineCols) / columns;
+  var dispPos = this._getDisplayPos(line);
+  var lineCols = dispPos.cols;
+  var lineRows = dispPos.rows;
 
   // cursor position
   var cursorPos = this._getCursorPos();
@@ -243,11 +282,11 @@ Interface.prototype._refreshLine = function() {
   exports.clearScreenDown(this.output);
 
   // Write the prompt and the current buffer content.
-  this.output.write(line);
+  this._writeToOutput(line);
 
   // Force terminal to allocate a new line
   if (lineCols === 0) {
-    this.output.write(' ');
+    this._writeToOutput(' ');
   }
 
   // Move cursor to original position.
@@ -264,10 +303,10 @@ Interface.prototype._refreshLine = function() {
 
 Interface.prototype.close = function() {
   if (this.closed) return;
+  this.pause();
   if (this.terminal) {
     this._setRawMode(false);
   }
-  this.pause();
   this.closed = true;
   this.emit('close');
 };
@@ -278,6 +317,7 @@ Interface.prototype.pause = function() {
   this.input.pause();
   this.paused = true;
   this.emit('pause');
+  return this;
 };
 
 
@@ -286,6 +326,7 @@ Interface.prototype.resume = function() {
   this.input.resume();
   this.paused = false;
   this.emit('resume');
+  return this;
 };
 
 
@@ -295,27 +336,31 @@ Interface.prototype.write = function(d, key) {
 };
 
 // \r\n, \n, or \r followed by something other than \n
-var lineEnding = /\r?\n|\r(?!\n)/;
+const lineEnding = /\r?\n|\r(?!\n)/;
 Interface.prototype._normalWrite = function(b) {
   if (b === undefined) {
     return;
   }
   var string = this._decoder.write(b);
-  if (this._sawReturn) {
+  if (this._sawReturnAt &&
+      Date.now() - this._sawReturnAt <= this.crlfDelay) {
     string = string.replace(/^\n/, '');
-    this._sawReturn = false;
+    this._sawReturnAt = 0;
   }
 
+  // Run test() on the new string chunk, not on the entire line buffer.
+  var newPartContainsEnding = lineEnding.test(string);
+
   if (this._line_buffer) {
     string = this._line_buffer + string;
     this._line_buffer = null;
   }
-  if (lineEnding.test(string)) {
-    this._sawReturn = /\r$/.test(string);
+  if (newPartContainsEnding) {
+    this._sawReturnAt = string.endsWith('\r') ? Date.now() : 0;
 
     // got one or more newlines; process into "line" events
     var lines = string.split(lineEnding);
-    // either '' or (concievably) the unfinished portion of the next line
+    // either '' or (conceivably) the unfinished portion of the next line
     string = lines.pop();
     this._line_buffer = string;
     lines.forEach(function(line) {
@@ -328,9 +373,6 @@ Interface.prototype._normalWrite = function(b) {
 };
 
 Interface.prototype._insertString = function(c) {
-  //BUG: Problem when adding tabs with following content.
-  //     Perhaps the bug is in _refreshLine(). Not sure.
-  //     A hack would be to insert spaces instead of literal '\t'.
   if (this.cursor < this.line.length) {
     var beg = this.line.slice(0, this.cursor);
     var end = this.line.slice(this.cursor, this.line.length);
@@ -344,7 +386,7 @@ Interface.prototype._insertString = function(c) {
     if (this._getCursorPos().cols === 0) {
       this._refreshLine();
     } else {
-      this.output.write(c);
+      this._writeToOutput(c);
     }
 
     // a hack to get the line refreshed if it's needed
@@ -360,22 +402,25 @@ Interface.prototype._tabComplete = function() {
     self.resume();
 
     if (err) {
-      // XXX Log it somewhere?
+      debug('tab completion error %j', err);
       return;
     }
 
-    var completions = rv[0],
-        completeOn = rv[1];  // the text that was completed
+    const completions = rv[0];
+    const completeOn = rv[1];  // the text that was completed
     if (completions && completions.length) {
       // Apply/show completions.
       if (completions.length === 1) {
         self._insertString(completions[0].slice(completeOn.length));
       } else {
-        self.output.write('\r\n');
+        self._writeToOutput('\r\n');
         var width = completions.reduce(function(a, b) {
           return a.length > b.length ? a : b;
         }).length + 2;  // 2 space padding
-        var maxColumns = Math.floor(self.columns / width) || 1;
+        var maxColumns = Math.floor(self.columns / width);
+        if (!maxColumns || maxColumns === Infinity) {
+          maxColumns = 1;
+        }
         var group = [], c;
         for (var i = 0, compLen = completions.length; i < compLen; i++) {
           c = completions[i];
@@ -404,7 +449,7 @@ Interface.prototype._tabComplete = function() {
 
 // this = Interface instance
 function handleGroup(self, group, width, maxColumns) {
-  if (group.length == 0) {
+  if (group.length === 0) {
     return;
   }
   var minRows = Math.ceil(group.length / maxColumns);
@@ -415,28 +460,28 @@ function handleGroup(self, group, width, maxColumns) {
         break;
       }
       var item = group[idx];
-      self.output.write(item);
+      self._writeToOutput(item);
       if (col < maxColumns - 1) {
         for (var s = 0, itemLen = item.length; s < width - itemLen;
              s++) {
-          self.output.write(' ');
+          self._writeToOutput(' ');
         }
       }
     }
-    self.output.write('\r\n');
+    self._writeToOutput('\r\n');
   }
-  self.output.write('\r\n');
+  self._writeToOutput('\r\n');
 }
 
 function commonPrefix(strings) {
-  if (!strings || strings.length == 0) {
+  if (!strings || strings.length === 0) {
     return '';
   }
   var sorted = strings.slice().sort();
   var min = sorted[0];
   var max = sorted[sorted.length - 1];
   for (var i = 0, len = min.length; i < len; i++) {
-    if (min[i] != max[i]) {
+    if (min[i] !== max[i]) {
       return min.slice(0, i);
     }
   }
@@ -518,7 +563,7 @@ Interface.prototype._deleteLineRight = function() {
 
 Interface.prototype.clearLine = function() {
   this._moveCursor(+Infinity);
-  this.output.write('\r\n');
+  this._writeToOutput('\r\n');
   this.line = '';
   this.cursor = 0;
   this.prevRows = 0;
@@ -559,12 +604,53 @@ Interface.prototype._historyPrev = function() {
 };
 
 
+// Returns the last character's display position of the given string
+Interface.prototype._getDisplayPos = function(str) {
+  var offset = 0;
+  var col = this.columns;
+  var row = 0;
+  var code;
+  str = stripVTControlCharacters(str);
+  for (var i = 0, len = str.length; i < len; i++) {
+    code = str.codePointAt(i);
+    if (code >= 0x10000) { // surrogates
+      i++;
+    }
+    if (code === 0x0a) { // new line \n
+      offset = 0;
+      row += 1;
+      continue;
+    }
+    if (isFullWidthCodePoint(code)) {
+      if ((offset + 1) % col === 0) {
+        offset++;
+      }
+      offset += 2;
+    } else {
+      offset++;
+    }
+  }
+  var cols = offset % col;
+  var rows = row + (offset - cols) / col;
+  return {cols: cols, rows: rows};
+};
+
+
 // Returns current cursor's position and line
 Interface.prototype._getCursorPos = function() {
   var columns = this.columns;
-  var cursorPos = this.cursor + this._promptLength;
-  var cols = cursorPos % columns;
-  var rows = (cursorPos - cols) / columns;
+  var strBeforeCursor = this._prompt + this.line.substring(0, this.cursor);
+  var dispPos = this._getDisplayPos(stripVTControlCharacters(strBeforeCursor));
+  var cols = dispPos.cols;
+  var rows = dispPos.rows;
+  // If the cursor is on a full-width character which steps over the line,
+  // move the cursor to the beginning of the next line.
+  if (cols + 1 === columns &&
+      this.cursor < this.line.length &&
+      isFullWidthCodePoint(this.line.codePointAt(this.cursor))) {
+    rows++;
+    cols = 0;
+  }
   return {cols: cols, rows: rows};
 };
 
@@ -578,13 +664,24 @@ Interface.prototype._moveCursor = function(dx) {
 
   // bounds check
   if (this.cursor < 0) this.cursor = 0;
-  if (this.cursor > this.line.length) this.cursor = this.line.length;
+  else if (this.cursor > this.line.length) this.cursor = this.line.length;
 
   var newPos = this._getCursorPos();
 
   // check if cursors are in the same line
   if (oldPos.rows === newPos.rows) {
-    exports.moveCursor(this.output, this.cursor - oldcursor, 0);
+    var diffCursor = this.cursor - oldcursor;
+    var diffWidth;
+    if (diffCursor < 0) {
+      diffWidth = -getStringWidth(
+          this.line.substring(this.cursor, oldcursor)
+          );
+    } else if (diffCursor > 0) {
+      diffWidth = getStringWidth(
+          this.line.substring(this.cursor, oldcursor)
+          );
+    }
+    exports.moveCursor(this.output, diffWidth, 0);
     this.prevRows = newPos.rows;
   } else {
     this._refreshLine();
@@ -597,7 +694,7 @@ Interface.prototype._ttyWrite = function(s, key) {
   key = key || {};
 
   // Ignore escape key - Fixes #2876
-  if (key.name == 'escape') return;
+  if (key.name === 'escape') return;
 
   if (key.ctrl && key.shift) {
     /* Control and shift pressed */
@@ -616,7 +713,7 @@ Interface.prototype._ttyWrite = function(s, key) {
 
     switch (key.name) {
       case 'c':
-        if (EventEmitter.listenerCount(this, 'SIGINT') > 0) {
+        if (this.listenerCount('SIGINT') > 0) {
           this.emit('SIGINT');
         } else {
           // This readline instance is finished
@@ -678,8 +775,8 @@ Interface.prototype._ttyWrite = function(s, key) {
         break;
 
       case 'z':
-        if (process.platform == 'win32') break;
-        if (EventEmitter.listenerCount(this, 'SIGTSTP') > 0) {
+        if (process.platform === 'win32') break;
+        if (this.listenerCount('SIGTSTP') > 0) {
           this.emit('SIGTSTP');
         } else {
           process.once('SIGCONT', (function(self) {
@@ -691,7 +788,7 @@ Interface.prototype._ttyWrite = function(s, key) {
                 self.pause();
                 self.emit('SIGCONT');
               }
-              // explictly re-enable "raw mode" and move the cursor to
+              // explicitly re-enable "raw mode" and move the cursor to
               // the correct position.
               // See https://github.com/joyent/node/issues/3295.
               self._setRawMode(true);
@@ -712,10 +809,6 @@ Interface.prototype._ttyWrite = function(s, key) {
         this._deleteWordRight();
         break;
 
-      case 'backspace':
-        this._deleteWordLeft();
-        break;
-
       case 'left':
         this._wordLeft();
         break;
@@ -751,20 +844,22 @@ Interface.prototype._ttyWrite = function(s, key) {
     /* No modifier keys used */
 
     // \r bookkeeping is only relevant if a \n comes right after.
-    if (this._sawReturn && key.name !== 'enter')
-      this._sawReturn = false;
+    if (this._sawReturnAt && key.name !== 'enter')
+      this._sawReturnAt = 0;
 
     switch (key.name) {
       case 'return':  // carriage return, i.e. \r
-        this._sawReturn = true;
+        this._sawReturnAt = Date.now();
         this._line();
         break;
 
       case 'enter':
-        if (this._sawReturn)
-          this._sawReturn = false;
-        else
+        // When key interval > crlfDelay
+        if (this._sawReturnAt === 0 ||
+            Date.now() - this._sawReturnAt > this.crlfDelay) {
           this._line();
+        }
+        this._sawReturnAt = 0;
         break;
 
       case 'backspace':
@@ -775,10 +870,6 @@ Interface.prototype._ttyWrite = function(s, key) {
         this._deleteRight();
         break;
 
-      case 'tab': // tab completion
-        this._tabComplete();
-        break;
-
       case 'left':
         this._moveCursor(-1);
         break;
@@ -803,8 +894,16 @@ Interface.prototype._ttyWrite = function(s, key) {
         this._historyNext();
         break;
 
+      case 'tab':
+        // If tab completion enabled, do that...
+        if (typeof this.completer === 'function' && this.isCompletionEnabled) {
+          this._tabComplete();
+          break;
+        }
+        // falls through
+
       default:
-        if (Buffer.isBuffer(s))
+        if (s instanceof Buffer)
           s = s.toString('utf-8');
 
         if (s) {
@@ -824,20 +923,61 @@ Interface.prototype._ttyWrite = function(s, key) {
 exports.Interface = Interface;
 
 
-
 /**
  * accepts a readable Stream instance and makes it emit "keypress" events
  */
 
-function emitKeypressEvents(stream) {
-  if (stream._keypressDecoder) return;
+const KEYPRESS_DECODER = Symbol('keypress-decoder');
+const ESCAPE_DECODER = Symbol('escape-decoder');
+
+// GNU readline library - keyseq-timeout is 500ms (default)
+const ESCAPE_CODE_TIMEOUT = 500;
+
+function emitKeypressEvents(stream, iface) {
+  if (stream[KEYPRESS_DECODER]) return;
   var StringDecoder = require('string_decoder').StringDecoder; // lazy load
-  stream._keypressDecoder = new StringDecoder('utf8');
+  stream[KEYPRESS_DECODER] = new StringDecoder('utf8');
+
+  stream[ESCAPE_DECODER] = emitKeys(stream);
+  stream[ESCAPE_DECODER].next();
+
+  const escapeCodeTimeout = () => stream[ESCAPE_DECODER].next('');
+  let timeoutId;
 
   function onData(b) {
-    if (EventEmitter.listenerCount(stream, 'keypress') > 0) {
-      var r = stream._keypressDecoder.write(b);
-      if (r) emitKey(stream, r);
+    if (stream.listenerCount('keypress') > 0) {
+      var r = stream[KEYPRESS_DECODER].write(b);
+      if (r) {
+        clearTimeout(timeoutId);
+
+        if (iface) {
+          iface._sawKeyPress = r.length === 1;
+        }
+
+        for (var i = 0; i < r.length; i++) {
+          if (r[i] === '\t' && typeof r[i + 1] === 'string' && iface) {
+            iface.isCompletionEnabled = false;
+          }
+
+          try {
+            stream[ESCAPE_DECODER].next(r[i]);
+            // Escape letter at the tail position
+            if (r[i] === '\x1b' && i + 1 === r.length) {
+              timeoutId = setTimeout(escapeCodeTimeout, ESCAPE_CODE_TIMEOUT);
+            }
+          } catch (err) {
+            // if the generator throws (it could happen in the `keypress`
+            // event), we need to restart it.
+            stream[ESCAPE_DECODER] = emitKeys(stream);
+            stream[ESCAPE_DECODER].next();
+            throw err;
+          } finally {
+            if (iface) {
+              iface.isCompletionEnabled = true;
+            }
+          }
+        }
+      }
     } else {
       // Nobody's watching anyway
       stream.removeListener('data', onData);
@@ -846,13 +986,13 @@ function emitKeypressEvents(stream) {
   }
 
   function onNewListener(event) {
-    if (event == 'keypress') {
+    if (event === 'keypress') {
       stream.on('data', onData);
       stream.removeListener('newListener', onNewListener);
     }
   }
 
-  if (EventEmitter.listenerCount(stream, 'keypress') > 0) {
+  if (stream.listenerCount('keypress') > 0) {
     stream.on('data', onData);
   } else {
     stream.on('newListener', onNewListener);
@@ -860,253 +1000,20 @@ function emitKeypressEvents(stream) {
 }
 exports.emitKeypressEvents = emitKeypressEvents;
 
-/*
-  Some patterns seen in terminal key escape codes, derived from combos seen
-  at http://www.midnight-commander.org/browser/lib/tty/key.c
-
-  ESC letter
-  ESC [ letter
-  ESC [ modifier letter
-  ESC [ 1 ; modifier letter
-  ESC [ num char
-  ESC [ num ; modifier char
-  ESC O letter
-  ESC O modifier letter
-  ESC O 1 ; modifier letter
-  ESC N letter
-  ESC [ [ num ; modifier char
-  ESC [ [ 1 ; modifier letter
-  ESC ESC [ num char
-  ESC ESC O letter
-
-  - char is usually ~ but $ and ^ also happen with rxvt
-  - modifier is 1 +
-                (shift     * 1) +
-                (left_alt  * 2) +
-                (ctrl      * 4) +
-                (right_alt * 8)
-  - two leading ESCs apparently mean the same as one leading ESC
-*/
-
-// Regexes used for ansi escape code splitting
-var metaKeyCodeRe = /^(?:\x1b)([a-zA-Z0-9])$/;
-var functionKeyCodeRe =
-    /^(?:\x1b+)(O|N|\[|\[\[)(?:(\d+)(?:;(\d+))?([~^$])|(?:1;)?(\d+)?([a-zA-Z]))/;
-
-function emitKey(stream, s) {
-  var ch,
-      key = {
-        name: undefined,
-        ctrl: false,
-        meta: false,
-        shift: false
-      },
-      parts;
-
-  if (Buffer.isBuffer(s)) {
-    if (s[0] > 127 && s[1] === undefined) {
-      s[0] -= 128;
-      s = '\x1b' + s.toString(stream.encoding || 'utf-8');
-    } else {
-      s = s.toString(stream.encoding || 'utf-8');
-    }
-  }
-
-  key.sequence = s;
-
-  if (s === '\r') {
-    // carriage return
-    key.name = 'return';
-
-  } else if (s === '\n') {
-    // enter, should have been called linefeed
-    key.name = 'enter';
-
-  } else if (s === '\t') {
-    // tab
-    key.name = 'tab';
-
-  } else if (s === '\b' || s === '\x7f' ||
-             s === '\x1b\x7f' || s === '\x1b\b') {
-    // backspace or ctrl+h
-    key.name = 'backspace';
-    key.meta = (s.charAt(0) === '\x1b');
-
-  } else if (s === '\x1b' || s === '\x1b\x1b') {
-    // escape key
-    key.name = 'escape';
-    key.meta = (s.length === 2);
-
-  } else if (s === ' ' || s === '\x1b ') {
-    key.name = 'space';
-    key.meta = (s.length === 2);
-
-  } else if (s <= '\x1a') {
-    // ctrl+letter
-    key.name = String.fromCharCode(s.charCodeAt(0) + 'a'.charCodeAt(0) - 1);
-    key.ctrl = true;
-
-  } else if (s.length === 1 && s >= 'a' && s <= 'z') {
-    // lowercase letter
-    key.name = s;
-
-  } else if (s.length === 1 && s >= 'A' && s <= 'Z') {
-    // shift+letter
-    key.name = s.toLowerCase();
-    key.shift = true;
-
-  } else if (parts = metaKeyCodeRe.exec(s)) {
-    // meta+character key
-    key.name = parts[1].toLowerCase();
-    key.meta = true;
-    key.shift = /^[A-Z]$/.test(parts[1]);
-
-  } else if (parts = functionKeyCodeRe.exec(s)) {
-    // ansi escape sequence
-
-    // reassemble the key code leaving out leading \x1b's,
-    // the modifier key bitflag and any meaningless "1;" sequence
-    var code = (parts[1] || '') + (parts[2] || '') +
-               (parts[4] || '') + (parts[6] || ''),
-        modifier = (parts[3] || parts[5] || 1) - 1;
-
-    // Parse the key modifier
-    key.ctrl = !!(modifier & 4);
-    key.meta = !!(modifier & 10);
-    key.shift = !!(modifier & 1);
-    key.code = code;
-
-    // Parse the key itself
-    switch (code) {
-      /* xterm/gnome ESC O letter */
-      case 'OP': key.name = 'f1'; break;
-      case 'OQ': key.name = 'f2'; break;
-      case 'OR': key.name = 'f3'; break;
-      case 'OS': key.name = 'f4'; break;
-
-      /* xterm/rxvt ESC [ number ~ */
-      case '[11~': key.name = 'f1'; break;
-      case '[12~': key.name = 'f2'; break;
-      case '[13~': key.name = 'f3'; break;
-      case '[14~': key.name = 'f4'; break;
-
-      /* from Cygwin and used in libuv */
-      case '[[A': key.name = 'f1'; break;
-      case '[[B': key.name = 'f2'; break;
-      case '[[C': key.name = 'f3'; break;
-      case '[[D': key.name = 'f4'; break;
-      case '[[E': key.name = 'f5'; break;
-
-      /* common */
-      case '[15~': key.name = 'f5'; break;
-      case '[17~': key.name = 'f6'; break;
-      case '[18~': key.name = 'f7'; break;
-      case '[19~': key.name = 'f8'; break;
-      case '[20~': key.name = 'f9'; break;
-      case '[21~': key.name = 'f10'; break;
-      case '[23~': key.name = 'f11'; break;
-      case '[24~': key.name = 'f12'; break;
-
-      /* xterm ESC [ letter */
-      case '[A': key.name = 'up'; break;
-      case '[B': key.name = 'down'; break;
-      case '[C': key.name = 'right'; break;
-      case '[D': key.name = 'left'; break;
-      case '[E': key.name = 'clear'; break;
-      case '[F': key.name = 'end'; break;
-      case '[H': key.name = 'home'; break;
-
-      /* xterm/gnome ESC O letter */
-      case 'OA': key.name = 'up'; break;
-      case 'OB': key.name = 'down'; break;
-      case 'OC': key.name = 'right'; break;
-      case 'OD': key.name = 'left'; break;
-      case 'OE': key.name = 'clear'; break;
-      case 'OF': key.name = 'end'; break;
-      case 'OH': key.name = 'home'; break;
-
-      /* xterm/rxvt ESC [ number ~ */
-      case '[1~': key.name = 'home'; break;
-      case '[2~': key.name = 'insert'; break;
-      case '[3~': key.name = 'delete'; break;
-      case '[4~': key.name = 'end'; break;
-      case '[5~': key.name = 'pageup'; break;
-      case '[6~': key.name = 'pagedown'; break;
-
-      /* putty */
-      case '[[5~': key.name = 'pageup'; break;
-      case '[[6~': key.name = 'pagedown'; break;
-
-      /* rxvt */
-      case '[7~': key.name = 'home'; break;
-      case '[8~': key.name = 'end'; break;
-
-      /* rxvt keys with modifiers */
-      case '[a': key.name = 'up'; key.shift = true; break;
-      case '[b': key.name = 'down'; key.shift = true; break;
-      case '[c': key.name = 'right'; key.shift = true; break;
-      case '[d': key.name = 'left'; key.shift = true; break;
-      case '[e': key.name = 'clear'; key.shift = true; break;
-
-      case '[2$': key.name = 'insert'; key.shift = true; break;
-      case '[3$': key.name = 'delete'; key.shift = true; break;
-      case '[5$': key.name = 'pageup'; key.shift = true; break;
-      case '[6$': key.name = 'pagedown'; key.shift = true; break;
-      case '[7$': key.name = 'home'; key.shift = true; break;
-      case '[8$': key.name = 'end'; key.shift = true; break;
-
-      case 'Oa': key.name = 'up'; key.ctrl = true; break;
-      case 'Ob': key.name = 'down'; key.ctrl = true; break;
-      case 'Oc': key.name = 'right'; key.ctrl = true; break;
-      case 'Od': key.name = 'left'; key.ctrl = true; break;
-      case 'Oe': key.name = 'clear'; key.ctrl = true; break;
-
-      case '[2^': key.name = 'insert'; key.ctrl = true; break;
-      case '[3^': key.name = 'delete'; key.ctrl = true; break;
-      case '[5^': key.name = 'pageup'; key.ctrl = true; break;
-      case '[6^': key.name = 'pagedown'; key.ctrl = true; break;
-      case '[7^': key.name = 'home'; key.ctrl = true; break;
-      case '[8^': key.name = 'end'; key.ctrl = true; break;
-
-      /* misc. */
-      case '[Z': key.name = 'tab'; key.shift = true; break;
-      default: key.name = 'undefined'; break;
-
-    }
-  } else if (s.length > 1 && s[0] !== '\x1b') {
-    // Got a longer-than-one string of characters.
-    // Probably a paste, since it wasn't a control sequence.
-    Array.prototype.forEach.call(s, function(c) {
-      emitKey(stream, c);
-    });
-    return;
-  }
-
-  // Don't emit a key if no name was found
-  if (key.name === undefined) {
-    key = undefined;
-  }
-
-  if (s.length === 1) {
-    ch = s;
-  }
-
-  if (key || ch) {
-    stream.emit('keypress', ch, key);
-  }
-}
-
 
 /**
  * moves the cursor to the x and y coordinate on the given stream
  */
 
 function cursorTo(stream, x, y) {
+  if (stream === null || stream === undefined)
+    return;
+
   if (typeof x !== 'number' && typeof y !== 'number')
     return;
 
   if (typeof x !== 'number')
-    throw new Error("Can't set cursor row without also setting it's column");
+    throw new Error('Can\'t set cursor row without also setting it\'s column');
 
   if (typeof y !== 'number') {
     stream.write('\x1b[' + (x + 1) + 'G');
@@ -1122,6 +1029,9 @@ exports.cursorTo = cursorTo;
  */
 
 function moveCursor(stream, dx, dy) {
+  if (stream === null || stream === undefined)
+    return;
+
   if (dx < 0) {
     stream.write('\x1b[' + (-dx) + 'D');
   } else if (dx > 0) {
@@ -1145,6 +1055,9 @@ exports.moveCursor = moveCursor;
  */
 
 function clearLine(stream, dir) {
+  if (stream === null || stream === undefined)
+    return;
+
   if (dir < 0) {
     // to the beginning
     stream.write('\x1b[1K');
@@ -1164,6 +1077,47 @@ exports.clearLine = clearLine;
  */
 
 function clearScreenDown(stream) {
+  if (stream === null || stream === undefined)
+    return;
+
   stream.write('\x1b[0J');
 }
 exports.clearScreenDown = clearScreenDown;
+
+
+/**
+ * Returns the Unicode code point for the character at the
+ * given index in the given string. Similar to String.charCodeAt(),
+ * but this function handles surrogates (code point >= 0x10000).
+ */
+
+function codePointAt(str, index) {
+  var code = str.charCodeAt(index);
+  var low;
+  if (0xd800 <= code && code <= 0xdbff) { // High surrogate
+    low = str.charCodeAt(index + 1);
+    if (!isNaN(low)) {
+      code = 0x10000 + (code - 0xd800) * 0x400 + (low - 0xdc00);
+    }
+  }
+  return code;
+}
+exports.codePointAt = internalUtil.deprecate(
+  codePointAt,
+  'readline.codePointAt is deprecated. ' +
+  'Use String.prototype.codePointAt instead.');
+
+
+exports.getStringWidth = internalUtil.deprecate(
+  getStringWidth,
+  'getStringWidth is deprecated and will be removed.');
+
+
+exports.isFullWidthCodePoint = internalUtil.deprecate(
+  isFullWidthCodePoint,
+  'isFullWidthCodePoint is deprecated and will be removed.');
+
+
+exports.stripVTControlCharacters = internalUtil.deprecate(
+    stripVTControlCharacters,
+    'stripVTControlCharacters is deprecated and will be removed.');
