@@ -1,13 +1,12 @@
 'use strict';
 
 const common = require('../../common');
-const binding = require('./build/Release/binding');
+const binding = require(`./build/${common.buildType}/binding`);
 const assert = require('assert');
 
-const skipMessage =
-  '1..0 # Skipped: intensive toString tests due to memory confinements';
+const skipMessage = 'intensive toString tests due to memory confinements';
 if (!common.enoughTestMem) {
-  console.log(skipMessage);
+  common.skip(skipMessage);
   return;
 }
 
@@ -16,28 +15,28 @@ if (!common.enoughTestMem) {
 const kStringMaxLength = process.binding('buffer').kStringMaxLength;
 
 try {
-  var buf = new Buffer(kStringMaxLength + 1);
+  var buf = Buffer.allocUnsafe(kStringMaxLength + 1);
 } catch (e) {
   // If the exception is not due to memory confinement then rethrow it.
-  if (e.message !== 'Invalid array buffer length') throw (e);
-  console.log(skipMessage);
+  if (e.message !== 'Array buffer allocation failed') throw (e);
+  common.skip(skipMessage);
   return;
 }
 
 // Ensure we have enough memory available for future allocations to succeed.
 if (!binding.ensureAllocation(2 * kStringMaxLength)) {
-  console.log(skipMessage);
+  common.skip(skipMessage);
   return;
 }
 
 assert.throws(function() {
-  buf.toString('binary');
-}, /toString failed/);
+  buf.toString('latin1');
+}, /"toString\(\)" failed/);
 
-var maxString = buf.toString('binary', 1);
-assert.equal(maxString.length, kStringMaxLength);
+var maxString = buf.toString('latin1', 1);
+assert.strictEqual(maxString.length, kStringMaxLength);
 // Free the memory early instead of at the end of the next assignment
 maxString = undefined;
 
-maxString = buf.toString('binary', 0, kStringMaxLength);
-assert.equal(maxString.length, kStringMaxLength);
+maxString = buf.toString('latin1', 0, kStringMaxLength);
+assert.strictEqual(maxString.length, kStringMaxLength);
