@@ -28,7 +28,7 @@ assert.doesNotThrow(makeBlock(a.ok, true),
 assert.doesNotThrow(makeBlock(a.ok, 'test'), 'ok(\'test\')');
 
 assert.throws(makeBlock(a.equal, true, false),
-                a.AssertionError, 'equal(true, false)');
+              a.AssertionError, 'equal(true, false)');
 
 assert.doesNotThrow(makeBlock(a.equal, null, null),
                     'equal(null, null)');
@@ -55,35 +55,58 @@ assert.throws(makeBlock(a.strictEqual, 2, '2'),
 assert.throws(makeBlock(a.strictEqual, null, undefined),
               a.AssertionError, 'strictEqual(null, undefined)');
 
+assert.throws(makeBlock(a.notStrictEqual, 2, 2),
+              a.AssertionError, 'notStrictEqual(2, 2)');
+
 assert.doesNotThrow(makeBlock(a.notStrictEqual, 2, '2'),
                     'notStrictEqual(2, \'2\')');
 
-// deepEquals joy!
+// deepEqual joy!
 // 7.2
 assert.doesNotThrow(makeBlock(a.deepEqual, new Date(2000, 3, 14),
-                    new Date(2000, 3, 14)),
+                              new Date(2000, 3, 14)),
                     'deepEqual(new Date(2000, 3, 14), new Date(2000, 3, 14))');
 
 assert.throws(makeBlock(a.deepEqual, new Date(), new Date(2000, 3, 14)),
               a.AssertionError,
               'deepEqual(new Date(), new Date(2000, 3, 14))');
 
+assert.throws(
+  makeBlock(a.notDeepEqual, new Date(2000, 3, 14), new Date(2000, 3, 14)),
+  a.AssertionError,
+  'notDeepEqual(new Date(2000, 3, 14), new Date(2000, 3, 14))'
+);
+
+assert.doesNotThrow(makeBlock(
+  a.notDeepEqual,
+  new Date(),
+  new Date(2000, 3, 14)),
+                    'notDeepEqual(new Date(), new Date(2000, 3, 14))'
+);
+
 // 7.3
 assert.doesNotThrow(makeBlock(a.deepEqual, /a/, /a/));
 assert.doesNotThrow(makeBlock(a.deepEqual, /a/g, /a/g));
 assert.doesNotThrow(makeBlock(a.deepEqual, /a/i, /a/i));
 assert.doesNotThrow(makeBlock(a.deepEqual, /a/m, /a/m));
 assert.doesNotThrow(makeBlock(a.deepEqual, /a/igm, /a/igm));
-assert.throws(makeBlock(a.deepEqual, /ab/, /a/));
-assert.throws(makeBlock(a.deepEqual, /a/g, /a/));
-assert.throws(makeBlock(a.deepEqual, /a/i, /a/));
-assert.throws(makeBlock(a.deepEqual, /a/m, /a/));
-assert.throws(makeBlock(a.deepEqual, /a/igm, /a/im));
+assert.throws(makeBlock(a.deepEqual, /ab/, /a/),
+              /^AssertionError: \/ab\/ deepEqual \/a\/$/);
+assert.throws(makeBlock(a.deepEqual, /a/g, /a/),
+              /^AssertionError: \/a\/g deepEqual \/a\/$/);
+assert.throws(makeBlock(a.deepEqual, /a/i, /a/),
+              /^AssertionError: \/a\/i deepEqual \/a\/$/);
+assert.throws(makeBlock(a.deepEqual, /a/m, /a/),
+              /^AssertionError: \/a\/m deepEqual \/a\/$/);
+assert.throws(makeBlock(a.deepEqual, /a/igm, /a/im),
+              /^AssertionError: \/a\/gim deepEqual \/a\/im$/);
 
 {
-  const re1 = /a/;
+  const re1 = /a/g;
   re1.lastIndex = 3;
-  assert.throws(makeBlock(a.deepEqual, re1, /a/));
+
+  assert.throws(makeBlock(a.deepEqual, re1, /a/g),
+                /^AssertionError: \/a\/g deepEqual \/a\/g$/);
 }
 
 
@@ -161,15 +184,31 @@ assert.doesNotThrow(makeBlock(a.deepEqual, new Number(1), {}),
 assert.doesNotThrow(makeBlock(a.deepEqual, new Boolean(true), {}),
                     a.AssertionError);
 
-//deepStrictEqual
-assert.doesNotThrow(makeBlock(a.deepStrictEqual, new Date(2000, 3, 14),
-                    new Date(2000, 3, 14)),
-                    'deepStrictEqual(new Date(2000, 3, 14),\
-                    new Date(2000, 3, 14))');
+// same number of keys but different key names
+assert.throws(makeBlock(a.deepEqual, {a: 1}, {b: 1}), a.AssertionError);
 
-assert.throws(makeBlock(a.deepStrictEqual, new Date(), new Date(2000, 3, 14)),
-              a.AssertionError,
-              'deepStrictEqual(new Date(), new Date(2000, 3, 14))');
+//deepStrictEqual
+assert.doesNotThrow(
+  makeBlock(a.deepStrictEqual, new Date(2000, 3, 14), new Date(2000, 3, 14)),
+  'deepStrictEqual(new Date(2000, 3, 14), new Date(2000, 3, 14))'
+);
+
+assert.throws(
+  makeBlock(a.deepStrictEqual, new Date(), new Date(2000, 3, 14)),
+  a.AssertionError,
+  'deepStrictEqual(new Date(), new Date(2000, 3, 14))'
+);
+
+assert.throws(
+  makeBlock(a.notDeepStrictEqual, new Date(2000, 3, 14), new Date(2000, 3, 14)),
+  a.AssertionError,
+  'notDeepStrictEqual(new Date(2000, 3, 14), new Date(2000, 3, 14))'
+);
+
+assert.doesNotThrow(
+  makeBlock(a.notDeepStrictEqual, new Date(), new Date(2000, 3, 14)),
+  'notDeepStrictEqual(new Date(), new Date(2000, 3, 14))'
+);
 
 // 7.3 - strict
 assert.doesNotThrow(makeBlock(a.deepStrictEqual, /a/, /a/));
@@ -177,11 +216,26 @@ assert.doesNotThrow(makeBlock(a.deepStrictEqual, /a/g, /a/g));
 assert.doesNotThrow(makeBlock(a.deepStrictEqual, /a/i, /a/i));
 assert.doesNotThrow(makeBlock(a.deepStrictEqual, /a/m, /a/m));
 assert.doesNotThrow(makeBlock(a.deepStrictEqual, /a/igm, /a/igm));
-assert.throws(makeBlock(a.deepStrictEqual, /ab/, /a/));
-assert.throws(makeBlock(a.deepStrictEqual, /a/g, /a/));
-assert.throws(makeBlock(a.deepStrictEqual, /a/i, /a/));
-assert.throws(makeBlock(a.deepStrictEqual, /a/m, /a/));
-assert.throws(makeBlock(a.deepStrictEqual, /a/igm, /a/im));
+assert.throws(
+  makeBlock(a.deepStrictEqual, /ab/, /a/),
+  /^AssertionError: \/ab\/ deepStrictEqual \/a\/$/
+);
+assert.throws(
+  makeBlock(a.deepStrictEqual, /a/g, /a/),
+  /^AssertionError: \/a\/g deepStrictEqual \/a\/$/
+);
+assert.throws(
+  makeBlock(a.deepStrictEqual, /a/i, /a/),
+  /^AssertionError: \/a\/i deepStrictEqual \/a\/$/
+);
+assert.throws(
+  makeBlock(a.deepStrictEqual, /a/m, /a/),
+  /^AssertionError: \/a\/m deepStrictEqual \/a\/$/
+);
+assert.throws(
+  makeBlock(a.deepStrictEqual, /a/igm, /a/im),
+  /^AssertionError: \/a\/gim deepStrictEqual \/a\/im$/
+);
 
 {
   const re1 = /a/;
@@ -326,14 +380,15 @@ try {
 assert.equal(true, threw,
              'a.doesNotThrow is not catching type matching errors');
 
-assert.throws(function() {assert.ifError(new Error('test error'));});
-assert.doesNotThrow(function() {assert.ifError(null);});
-assert.doesNotThrow(function() {assert.ifError();});
+assert.throws(function() { assert.ifError(new Error('test error')); },
+              /^Error: test error$/);
+assert.doesNotThrow(function() { assert.ifError(null); });
+assert.doesNotThrow(function() { assert.ifError(); });
 
 assert.throws(() => {
   assert.doesNotThrow(makeBlock(thrower, Error), 'user message');
 }, /Got unwanted exception. user message/,
-   'a.doesNotThrow ignores user message');
+              'a.doesNotThrow ignores user message');
 
 // make sure that validating using constructor really works
 threw = false;
@@ -375,31 +430,53 @@ try {
 } catch (e) {
   threw = true;
   assert(e instanceof AnotherErrorType,
-    `expected AnotherErrorType, received ${e}`);
+         `expected AnotherErrorType, received ${e}`);
 }
 
 assert.ok(threw);
 
-// GH-207. Make sure deepEqual doesn't loop forever on circular refs
-var b = {};
-b.b = b;
+// https://github.com/nodejs/node/issues/6416
+// Make sure circular refs don't throw.
+{
+  const b = {};
+  b.b = b;
 
-var c = {};
-c.b = c;
+  const c = {};
+  c.b = c;
 
-var gotError = false;
-try {
-  assert.deepEqual(b, c);
-} catch (e) {
-  gotError = true;
+  a.doesNotThrow(makeBlock(a.deepEqual, b, c));
+  a.doesNotThrow(makeBlock(a.deepStrictEqual, b, c));
+
+  const d = {};
+  d.a = 1;
+  d.b = d;
+
+  const e = {};
+  e.a = 1;
+  e.b = e.a;
+
+  a.throws(makeBlock(a.deepEqual, d, e), /AssertionError/);
+  a.throws(makeBlock(a.deepStrictEqual, d, e), /AssertionError/);
 }
-
 // GH-7178. Ensure reflexivity of deepEqual with `arguments` objects.
 var args = (function() { return arguments; })();
 a.throws(makeBlock(a.deepEqual, [], args));
 a.throws(makeBlock(a.deepEqual, args, []));
-assert.ok(gotError);
 
+// more checking that arguments objects are handled correctly
+{
+  const returnArguments = function() { return arguments; };
+
+  const someArgs = returnArguments('a');
+  const sameArgs = returnArguments('a');
+  const diffArgs = returnArguments('b');
+
+  a.throws(makeBlock(a.deepEqual, someArgs, ['a']));
+  a.throws(makeBlock(a.deepEqual, ['a'], someArgs));
+  a.throws(makeBlock(a.deepEqual, someArgs, {'0': 'a'}));
+  a.throws(makeBlock(a.deepEqual, someArgs, diffArgs));
+  a.doesNotThrow(makeBlock(a.deepEqual, someArgs, sameArgs));
+}
 
 var circular = {y: 1};
 circular.x = circular;
@@ -409,7 +486,7 @@ function testAssertionMessage(actual, expected) {
     assert.equal(actual, '');
   } catch (e) {
     assert.equal(e.toString(),
-        ['AssertionError:', expected, '==', '\'\''].join(' '));
+                 ['AssertionError:', expected, '==', '\'\''].join(' '));
     assert.ok(e.generatedMessage, 'Message not marked as generated');
   }
 }
@@ -435,7 +512,7 @@ testAssertionMessage({}, '{}');
 testAssertionMessage(circular, '{ y: 1, x: [Circular] }');
 testAssertionMessage({a: undefined, b: null}, '{ a: undefined, b: null }');
 testAssertionMessage({a: NaN, b: Infinity, c: -Infinity},
-    '{ a: NaN, b: Infinity, c: -Infinity }');
+                     '{ a: NaN, b: Infinity, c: -Infinity }');
 
 // #2893
 try {
@@ -461,7 +538,7 @@ try {
 } catch (e) {
   assert.equal(e.toString().split('\n')[0], 'AssertionError: oh no');
   assert.equal(e.generatedMessage, false,
-              'Message incorrectly marked as generated');
+               'Message incorrectly marked as generated');
 }
 
 // Verify that throws() and doesNotThrow() throw on non-function block
@@ -472,7 +549,8 @@ function testBlockTypeError(method, block) {
     method(block);
     threw = false;
   } catch (e) {
-    assert.equal(e.toString(), 'TypeError: block must be a function');
+    assert.equal(e.toString(),
+                 'TypeError: "block" argument must be a function');
   }
 
   assert.ok(threw);
@@ -501,4 +579,9 @@ testBlockTypeError(assert.doesNotThrow, undefined);
 assert.throws(() => { throw 'error'; }, (err) => err === 'error');
 assert.throws(() => { throw new Error(); }, (err) => err instanceof Error);
 
+// Long values should be truncated for display.
+assert.throws(() => {
+  assert.strictEqual('A'.repeat(1000), '');
+}, new RegExp(`^AssertionError: '${'A'.repeat(127)} === ''$`));
+
 console.log('All OK');
