@@ -1,8 +1,8 @@
 'use strict';
 require('../common');
-var assert = require('assert');
+const assert = require('assert');
 
-var buffer = new Buffer([1, 2, 3, 4, 5]);
+const buffer = Buffer.from([1, 2, 3, 4, 5]);
 var arr;
 var b;
 
@@ -13,7 +13,7 @@ arr = [];
 for (b of buffer)
   arr.push(b);
 
-assert.deepEqual(arr, [1, 2, 3, 4, 5]);
+assert.deepStrictEqual(arr, [1, 2, 3, 4, 5]);
 
 
 // buffer iterators should be iterable
@@ -23,7 +23,7 @@ arr = [];
 for (b of buffer[Symbol.iterator]())
   arr.push(b);
 
-assert.deepEqual(arr, [1, 2, 3, 4, 5]);
+assert.deepStrictEqual(arr, [1, 2, 3, 4, 5]);
 
 
 // buffer#values() should return iterator for values
@@ -33,7 +33,7 @@ arr = [];
 for (b of buffer.values())
   arr.push(b);
 
-assert.deepEqual(arr, [1, 2, 3, 4, 5]);
+assert.deepStrictEqual(arr, [1, 2, 3, 4, 5]);
 
 
 // buffer#keys() should return iterator for keys
@@ -43,7 +43,7 @@ arr = [];
 for (b of buffer.keys())
   arr.push(b);
 
-assert.deepEqual(arr, [0, 1, 2, 3, 4]);
+assert.deepStrictEqual(arr, [0, 1, 2, 3, 4]);
 
 
 // buffer#entries() should return iterator for entries
@@ -53,7 +53,7 @@ arr = [];
 for (b of buffer.entries())
   arr.push(b);
 
-assert.deepEqual(arr, [
+assert.deepStrictEqual(arr, [
   [0, 1],
   [1, 2],
   [2, 3],
