@@ -1,7 +1,6 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
-
 var net = require('net');
 
 var s = new net.Stream();
@@ -11,6 +10,7 @@ var s = new net.Stream();
 
 s.server = new net.Server();
 s.server.connections = 10;
+s._server = s.server;
 
 assert.equal(10, s.server.connections);
 s.destroy();
@@ -20,8 +20,7 @@ assert.equal(9, s.server.connections);
 
 var SIZE = 2E6;
 var N = 10;
-var buf = new Buffer(SIZE);
-buf.fill(0x61); // 'a'
+var buf = Buffer.alloc(SIZE, 'a');
 
 var server = net.createServer(function(socket) {
   socket.setNoDelay();
@@ -37,15 +36,15 @@ var server = net.createServer(function(socket) {
   }
   socket.end();
 
-}).listen(common.PORT, function() {
-    var conn = net.connect(common.PORT);
-    conn.on('data', function(buf) {
-      conn.pause();
-      setTimeout(function() {
-        conn.destroy();
-      }, 20);
-    });
+}).listen(0, function() {
+  var conn = net.connect(this.address().port);
+  conn.on('data', function(buf) {
+    conn.pause();
+    setTimeout(function() {
+      conn.destroy();
+    }, 20);
   });
+});
 
 process.on('exit', function() {
   assert.equal(server.connections, 0);
