@@ -20,18 +20,32 @@ process.nextTick(function() {
   order.push('C');
 });
 
+function testNextTickWith(val) {
+  assert.throws(
+    function() {
+      process.nextTick(val);
+    },
+    TypeError
+  );
+}
+
+testNextTickWith(false);
+testNextTickWith(true);
+testNextTickWith(1);
+testNextTickWith('str');
+testNextTickWith({});
+testNextTickWith([]);
+
 process.on('uncaughtException', function() {
   if (!exceptionHandled) {
     exceptionHandled = true;
     order.push('B');
-  }
-  else {
+  } else {
     // If we get here then the first process.nextTick got called twice
     order.push('OOPS!');
   }
 });
 
 process.on('exit', function() {
-  assert.deepEqual(['A', 'B', 'C'], order);
+  assert.deepStrictEqual(['A', 'B', 'C'], order);
 });
-
